#lang racket/base
(require racket/contract racket/match)

(define (string-len-one? x)
  (and (string? x)
       (= 1 (string-length x))))
  
(provide/contract
 [tex-shortcut-table
  (listof (list/c string? string-len-one?))])

(define blackboard-bold
  (append
   (for*/list ([i (in-string "ABCDEFGHIJKLMNOPQRSTUVWXYZ")]
               [case (in-list '(lower upper))])
     (cond
       [(and (equal? case 'upper) (member i (string->list "CHNPQRZ")))
        (list (format "b~a" i)
              (match i
                [#\C "ℂ"]
                [#\H "ℍ"]
                [#\N "ℕ"]
                [#\P "ℙ"]
                [#\Q "ℚ"]
                [#\R "ℝ"]
                [#\Z "ℤ"]))]
       [else
        (define bo (match case
                     ['upper (- (char->integer #\𝔸) (char->integer #\A))]
                     ['lower (- (char->integer #\𝕒) (char->integer #\A))]))
        (define co (match case
                     ['upper 0]
                     ['lower (- (char->integer #\a) (char->integer #\A))]))
        (list (format "b~a" (integer->char (+ (char->integer i) co)))
              (string (integer->char (+ (char->integer i) bo))))]))
   (for/list ([i (in-inclusive-range 0 9)])
     (list (format "b~a" i)
           (string (integer->char (+ (char->integer #\𝟘) i)))))))

(define tex-shortcut-table
  (append
  '(("Downarrow" "⇓")
    ("nwarrow" "↖")
    ("downarrow" "↓")
    ("Rightarrow" "⇒")
    ("rightarrow" "→")
    ("rightharpoonup" "⇀")
    ("leftharpoonup" "↼")
    ("rightharpoondown" "⇁")
    ("leftharpoondown" "↽")
    ("rightleftharpoons" "⇌")
    ("mapsto" "↦")
    ("searrow" "↘")
    ("swarrow" "↙")
    ("leftarrow" "←")
    ("uparrow" "↑")
    ("Leftarrow" "⇐")
    ("longrightarrow" "⟶")
    ("Uparrow" "⇑")
    ("Leftrightarrow" "⇔")
    ("updownarrow" "↕")
    ("leftrightarrow" "↔")
    ("nearrow" "↗")
    ("Updownarrow" "⇕")
    ("leadsto" "↝")
   
    ;; arrows that didn't come out right in copy & paste
    ;←− \longleftarrow
    ;⇐= \Longleftarrow 
    ;← 􏰂 \hookleftarrow
    ;←→ \longleftrightarrow 
    ;􏰁 → \hookrightarrow
    ;⇐⇒ \Longleftrightarrow
    ;􏴲 \leadsto∗ 
    ;􏰃−→ \longmapsto
    ;=⇒ \Longrightarrow 
    ;􏰃→ \mapsto
    
    ("aleph" "א")
    ("prime" "′")
    ("emptyset" "∅")
    ("nabla" "∇")
    ("diamondsuit" "♦")
    ("spadesuit" "♠")
    ("clubsuit" "♣")
    ("heartsuit" "♥")
    ("sharp" "♯")
    ("flat" "♭")
    ("natural" "♮")
    ("surd" "√")
    ("neg" "¬")
    ("triangle" "△")
    ("forall" "∀")
    ("all" "∀")
    ("exists" "∃")
    ("infty" "∞")
    ("circ" "∘")
    ("partial" "∂")
    ("alpha" "α")
    ("theta" "θ")
    ("tau" "τ")
    ("beta" "β")
    ("vartheta" "ϑ")
    ("pi" "π")
    ("upsilon" "υ")
    ("gamma" "γ")
    ("varpi" "ϖ")
    ("phi" "ϕ")
    ("delta" "δ")
    ("kappa" "κ")
    ("rho" "ρ")
    ("varphi" "φ")
    ("epsilon" "ϵ")
    ("lambda" "λ")
    ("varrho" "ϱ")
    ("chi" "χ")
    ("varepsilon" "ε")
    ("mu" "μ")
    ("sigma" "σ")
    ("psi" "ψ")
    ("zeta" "ζ")
    ("nu" "ν")
    ("varsigma" "ς")
    ("omega" "ω")
    ("eta" "η")
    ("xi" "ξ")
    ("iota" "ι")
    ("Gamma" "Γ")
    ("Lambda" "Λ")
    ("Sigma" "Σ")
    ("Psi" "Ψ")
    ("Delta" "Δ")
    ("Xi" "Ξ")
    ("Upsilon" "Υ")
    ("Omega" "Ω")
    ("Theta" "Θ")
    ("Pi" "Π")
    ("Phi" "Φ")
    ("pm" "±")
    ("cap" "∩")
    ("diamond" "◇")
    ("lozenge" "◊")
    ("square" "□")
    ("oplus" "⊕")
    ("mp" "∓")
    ("cup" "∪")
    ("bigtriangleup" "△")
    ("ominus" "⊖")
    ("times" "×")
    ("uplus" "⊎")
    ("bigtriangledown" "▽")
    ("otimes" "⊗")
    ("div" "÷")
    ("sqcap" "⊓")
    ("triangleright" "▹")
    ("oslash" "⊘")
    ("ast" "∗")
    ("sqcup" "⊔")
    ("vee" "∨")
    ("wedge" "∧")
    ("lor" "∨")
    ("land" "∧")
    ("lnot" "¬")
    ("triangleleft" "◃")
    ("angle" "∠")
    ("odot" "⊙")
    ("star" "★")
    ("dagger" "†")
    ("bullet" "•")
    ("ddagger" "‡")
    ("wr" "≀")
    ("amalg" "⨿")
    ("leq" "≤")
    ("geq" "≥")
    ("equiv" "≡")
    ("models" "⊨")
    ("prec" "≺")
    ("succ" "≻")
    ("precdot" "⋖")
    ("succdot" "⋗")
    ("sim" "∼")
    ("perp" "⊥")
    ("bot" "⊥")
    ("top" "⊤")
    ("preceq" "≼")
    ("succeq" "≽")
    ("simeq" "≃")
    ("ll" "≪")
    ("gg" "≫")
    ("guillemetleft" "«")
    ("guillemetright" "»")
    ("asymp" "≍")
    ("parallel" "∥")
    ("subset" "⊂")
    ("supset" "⊃")
    ("approx" "≈")
    ("bowtie" "⋈")
    ("subseteq" "⊆")
    ("supseteq" "⊇")
    ("nsubseteq" "⊈")
    ("subsetneq" "⊊")
    ("cong" "≅")
    ("sqsubsetb" "⊏")
    ("sqsupsetb" "⊐")
    ("neq" #;"≠" "≠")
    ("smile" "⌣")
    ("sqsubseteq" "⊑")
    ("sqsupseteq" "⊒")
    ("doteq" "≐")
    ("frown" "⌢")
    ("in" "∈")
    ("ni" "∋")
    ("notin" "∉")
    ("propto" "∝")
    ("vdash" "⊢")
    ("dashv" "⊣")
    ("cdot" "·")
    ("sum" "∑")
    ("prod" "∏")
    ("coprod" "∐")
    
    ("int" "∫")
    ("iint" "∬")
    ("iiint" "∭")
    ("oint" "∮")
    ("oiint" "∯")
    ("oiiint" "∰")

    ("sqrt" "√")
    
    ("skull" "☠") 
    ("smiley" "☺")
    ("blacksmiley" "☻")
    ("frownie" "☹")

    ("Re" "ℜ")
    ("Im" "ℑ")
    ("S" "§")
    ("l" "ł")

    ("newpage" "\f")
    
    ("vdots" "⋮")
    ("ddots" "⋱")
    ("cdots" "⋯")
    ("hdots" "⋯")
    ("ldots" "…")
    
    ("langle" "⟨")
    ("rangle" "⟩")

    ("amp" "&")
    ("invamp" "⅋")

    ("multimap" "⊸")
    ("rightlollipop" "⊸")
    ("leftmultimap" "⟜")
    ("multimapinv" "⟜")
    ("leftlollipop" "⟜")
    )
  blackboard-bold))

(module+ test
  (define name-ht (make-hash))
  (define val-ht (make-hash))
  (for ([line (in-list tex-shortcut-table)])
    (match-define (list name obj) line)
    (hash-set! name-ht name (cons obj (hash-ref name-ht name '()))))
  (for ([(k v) (in-hash name-ht)])
    (unless (= 1 (length v))
      (eprintf "duplicate: ~s -> ~s\n" k v))))


